"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isTargetLabels = exports.isTargetEvent = void 0;
const core_1 = require("@actions/core");
const context_1 = require("./context");
const getBoolValue = (input) => !['false', '0', '', 'no', 'n'].includes(input.trim().toLowerCase());
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const isTargetEventName = (events, context, options) => {
    if ('*' in events) {
        return true;
    }
    if (!(options === null || options === void 0 ? void 0 : options.notCheckWorkflowRun) && !('workflow_run' in events) && 'workflow_run' === context.eventName) {
        events['workflow_run'] = '*';
    }
    if (!(options === null || options === void 0 ? void 0 : options.notCheckPrTarget) && 'pull_request' in events && !('pull_request_target' in events)) {
        events['pull_request_target'] = events['pull_request'];
    }
    return context.eventName in events;
};
// eslint-disable-next-line @typescript-eslint/no-explicit-any
const isTargetEventAction = (action, context, some = true) => {
    if (Array.isArray(action)) {
        if (some) {
            return action.some(item => isTargetEventAction(item, context, false));
        }
        return !action.some(item => !isTargetEventAction(item, context, false));
    }
    if (typeof action === 'function') {
        return action(context);
    }
    return '*' === action || context.payload.action === action;
};
/**
 * @param {object} targets targets
 * @param {Context} context context
 * @param {OptionType} options options
 * @return {boolean} is target event?
 */
const isTargetEvent = (targets, context, options) => // eslint-disable-line @typescript-eslint/no-explicit-any,@typescript-eslint/explicit-module-boundary-types
 {
    var _a;
    return getBoolValue(core_1.getInput('IGNORE_CONTEXT_CHECK')) ||
        (isTargetEventName(targets, context, options) && isTargetEventAction((_a = targets[context.eventName]) !== null && _a !== void 0 ? _a : targets['*'], context));
};
exports.isTargetEvent = isTargetEvent;
/**
 * @param {string[]} includes include labels
 * @param {string[]} excludes exclude labels
 * @param {Context} context context
 * @return {boolean} is target labels?
 */
const isTargetLabels = (includes, excludes, context) => {
    const labels = context_1.getLabels(context);
    if (false === labels) {
        return false;
    }
    return (!includes.length || !!labels.filter(label => includes.includes(label)).length) && !labels.filter(label => excludes.includes(label)).length;
};
exports.isTargetLabels = isTargetLabels;
